/********************************************************************\

  Name:         camac.h
  Created by:   Stefan Ritt

  Cotents:      MIDAS standard CAMAC function declarations

  Revision history
  ------------------------------------------------------------------
  date        by    modification
  ---------   ---   ------------------------------------------------
  27-MAR-98   SR    created

\********************************************************************/

/*------------------------------------------------------------------*/

/* functions using CNAF directly */

/* input */

INLINE void cami(int c, int n, int a, int f, unsigned short *d);
INLINE void cam16i(int c, int n, int a, int f, unsigned short *d);
INLINE void cam24i(int c, int n, int a, int f, unsigned long *d);
INLINE void cam16i_q(int c, int n, int a, int f, unsigned short *d, int *x, int *q);
INLINE void cam24i_q(int c, int n, int a, int f, unsigned long *d, int *x, int *q);
INLINE void cam16i_r(int c, int n, int a, int f, unsigned short *d, int r);
INLINE void cam24i_r(int c, int n, int a, int f, unsigned long *d, int r);
INLINE void cam16i_sa(int c, int n, int a, int f, unsigned short *d, int r);
INLINE void cam24i_sa(int c, int n, int a, int f, unsigned long *d, int r);
INLINE void cam16i_sn(int c, int n, int a, int f, unsigned short *d, int r);
INLINE void cam24i_sn(int c, int n, int a, int f, unsigned long *d, int r);

/* output */
INLINE void camo(int c, int n, int a, int f, unsigned short d);
INLINE void cam16o(int c, int n, int a, int f, unsigned short d);
INLINE void cam24o(int c, int n, int a, int f, unsigned long d);
INLINE void cam16o_q(int c, int n, int a, int f, unsigned short d, int *x, int *q);
INLINE void cam24o_q(int c, int n, int a, int f, unsigned long d, int *x, int *q);

/* control */
INLINE void camc(int c, int n, int a, int f);
INLINE void camc_sa(int c, int n, int a, int f, int r);
INLINE void camc_sn(int c, int n, int a, int f, int r);
INLINE void camq(int c, int n, int a, int f, int *q);

/* functions using external representation */

/* input */

INLINE void came_cn(int *ext, int b, int c, int n, int a);
INLINE void cam16ei(int ext, int f, unsigned short *d);
INLINE void cam24ei(int ext, int f, unsigned long *d);
INLINE void cam16ei_q(int ext, int f, unsigned short *d, int *x, int *q);
INLINE void cam24ei_q(int ext, int f, unsigned long *d, int *x, int *q);
INLINE void cam16ei_r(int ext, int f, unsigned short *d, int r);
INLINE void cam24ei_r(int ext, int f, unsigned long *d, int r);
INLINE void cam16ei_sa(int ext, int f, unsigned short *d, int r);
INLINE void cam24ei_sa(int ext, int f, unsigned long *d, int r);
INLINE void cam16ei_sn(int ext, int f, unsigned short *d, int r);
INLINE void cam24ei_sn(int ext, int f, unsigned long *d, int r);

INLINE void cam16eo(int ext, int f, unsigned short d);
INLINE void cam24eo(int ext, int f, unsigned long d);
INLINE void cam16eo_q(int ext, int f, unsigned short d, int *x, int *q);
INLINE void cam24eo_q(int ext, int f, unsigned long d, int *x, int *q);

INLINE void camec(int ext, int f);
INLINE void camec_sn(int ext, int f, int r);
INLINE void camec_sa(int ext, int f, int r);

/* general functions */

void cam_init(void);
void cam_exit(void);

INLINE void cam_set_inhibit(int c);
INLINE void cam_clear_inhibit(int c);
INLINE void cam_c_cycle(int c);
INLINE void cam_c_cycle(int c);

INLINE void cam_lam_enable(int c, int n);
INLINE void cam_lam_disable(int c, int n);
INLINE void cam_lam_read(int c, int *lam);

INLINE void cam_interrupt_attach(void (*func)());
INLINE void cam_interrupt_detach(void);
INLINE void cam_interrupt_enable(void);
INLINE void cam_interrupt_disable(void);
