/********************************************************************\

  Name:         hyt1331.h
  Created by:   Stefan Ritt

  Cotents:      CAMAC functions for HYTEC 1331 Turbo controller

  Revision history
  ------------------------------------------------------------------
  date        by    modification
  ---------   ---   ------------------------------------------------
  17-MAR-98   SR    created

\********************************************************************/

#include <conio.h>

/*------------------------------------------------------------------*/

/*
  Base address of PC card. Must match jumper setting:
  ===================================================

  LK1    LK2    ADDRESS

  IN     IN     0x200 - 0x23F
  OUT    IN     0x280 - 0x3BF
  IN     OUT    0x300 - 0x33F
  OUT    OUT    0x380 - 0x3BF
*/

#define CAMAC_BASE 0x300

/*
  Interrupt request selection. Must match jumper setting:
  =======================================================

  LINK IN       IRQ line    Software interrupt

   9            IRQ 5       INT 0x0D (13)
  10            IRQ 3       INT 0x0B (11)
  11            IRQ 9       INT 0x02 (Cascaded)
  12            IRQ 12      INT 0x02 (Cascaded)
*/

#define LAM_INT 0x0D

/*------------------------------------------------------------------*/

#ifndef CAMAC_BASE_DEFINED
#define CAMAC_BASE_DEFINED
unsigned short _cam_base = CAMAC_BASE;
#else
extern unsigned short _cam_base;
#endif

/*------------------------------------------------------------------*/

#define CAMAC_N(_n)  _outp((short)(_cam_base+8), (char) _n)

#define CAMAC_A(_a)  _outp((short)(_cam_base+6), (char) _a)

#define CAMAC_F(_f)  _outp((short)(_cam_base+10), (char) _f)

#define CAMAC_OB(_d) _outp((short)(_cam_base), (char) _d)

#define CAMAC_OW(_d) \
                     _outp((short)(_cam_base), (char) _d); \
                     _outp((short)(_cam_base+2), (char) ((_d) >> 8))

#define CAMAC_OD(_d) \
                     _outp((short)(_cam_base), (char) _d); \
                     _outp((short)(_cam_base+2), (char) ((_d) >> 8)) \
                     _outp((short)(_cam_base+4), (char) ((_d) >> 16))

#define CAMAC_S      _inp((short)(_cam_base+6))

#define CAMAC_Q(_x, _q)                 \
{ char _status;                         \
  _status = _inp((short)(_cam_base+6)); \
  *_q = _status & 1;                    \
  *_x = _status >> 7; }

/*------------------------------------------------------------------*/

#define CAMO(_n, _a, _f, _d) \
{ CAMAC_OW(_d);      \
  CAMAC_A(_a);       \
  CAMAC_N(_n);       \
  CAMAC_F(_f);       \
  CAMAC_S; }


#define CAMI(_n, _a, _f, _d) \
{ CAMAC_A(_a);       \
  CAMAC_N(_n);       \
  CAMAC_F(_f);       \
  CAMAC_S;           \
  *(_d) = (unsigned int) (_inp(_cam_base) + (_inp((short)(_cam_base+2)) << 8)); }

#define CAMI24(_n, _a, _f, _d) \
{ CAMAC_A(_a);       \
  CAMAC_N(_n);       \
  CAMAC_F(_f);       \
  CAMAC_S;           \
  *(_d) = (unsigned long) (_inp(_cam_base) + (_inp((short)(_cam_base+2)) << 8) + (_inp((short)(_cam_base+4))<<16));}

#define CAMQ(_n, _a, _f, _q)   \
{ CAMO(_n, _a, _f, 0);         \
  *_q = _inp((short)(_cam_base+6)) & 1; }

#define CAMIQ(_n, _a, _f, _d, _x, _q) \
{ CAMI(_n, _a, _f, _d)                \
  CAMAC_Q(_x, _q); }

#define CAMIQ24(_n, _a, _f, _d, _x, _q) \
{ CAMI24(_n, _a, _f, _d)                \
  CAMAC_Q(_x, _q); }

#define CAMOQ(_n, _a, _f, _d, _x, _q)   \
{ CAMO(_n, _a, _f, _d);                 \
  CAMAC_Q(_x, _q); }

#define CAMAC_INIT() \
  ss_directio_init(); \
  ss_directio_give_port(_cam_base, _cam_base+0x3F); \
  ss_directio_exit()

#define CAMAC_SET_CRATE(_crate) \
  _cam_base = CAMAC_BASE + 0x10*_crate; \

#define CAML(d) \
  { int s;      \
    s = CAMAC_S;  \
    if (s & 0x08) \
      *d = _inp((short)(_cam_base+8)); \
    else \
      *d = 0; }

#define CAMAC_LAM_RESET() _inp((short)(_cam_base+8))

#define CAMAC_LAM_ENABLE(n)   \
  _outp((short)(_cam_base+10), 64+n); \
  _outp((short)(_cam_base+10), 41)

#define CAMAC_SET_INHIBIT()   _outp((short)(_cam_base+10), 33)

#define CAMAC_CLEAR_INHIBIT() _outp((short)(_cam_base+10), 32)

#define CAMAC_C_CYCLE()       _outp((short)(_cam_base+10), 36)

#define CAMAC_Z_CYCLE()       _outp((short)(_cam_base+10), 34)

/*------------------------------------------------------------------*/

#define CAMAC_TEST void camo(n,a,f,d)  \
{ CAMO(n,a,f,d); }                     \
void cami(n,a,f) {                     \
  unsigned long d,x,q;                 \
  CAMIQ(n,a,f,&d,&x,&q);               \
  printf("d=%lX,X=%d,Q=%d\n",d,x,q); } \
void caml() {                          \
  unsigned long d;                     \
  CAML(&d);                            \
  printf("l=%lX\n",d); }               \
void set_inhibit(void)                 \
{ *((unsigned long *) 0xC0E7A4) = 0; } \
void clear_inhibit(void)               \
{ *((unsigned long *) 0xC0C7A4) = 0; } \
void c_cycle(void)                     \
{ *((unsigned long *) 0xC0D724) = 0; } \
void z_cycle(void)                     \
{ *((unsigned long *) 0xC0D720) = 0; } 


