/*-----------------------------------------------------------------------------
 * Copyright (c) 1996      TRIUMF Data Acquistion Group
 * Please leave this header in any reproduction of that distribution
 * 
 * TRIUMF Data Acquisition Group, 4004 Wesbrook Mall, Vancouver, B.C. V6T 2A3
 * Email: online@triumf.ca           Tel: (604) 222-1047  Fax: (604) 222-1074
 *         amaudruz@triumf.ca
 * ----------------------------------------------------------------------------
 *  
 * Description	: VME Macros and functions for the LRS1190 Dual port buffer
 *		  Display result on the VME console screen.
 *		  No reference to Midas.
 *
 * Application : VME + Vxworks
 *
 * Author:  Pierre-Andre Amaudruz Data Acquisition Group
 * 
 * Revision 1.0  1996/ Pierre	 Initial revision
 * Revision 1.1  1998/ Pierre	 Split include for specific hardware.
 * ---------------------------------------------------------------------------- */
/*------------------------------------------------------------------------------------*/
#include <stdio.h>

#if defined( _MSC_VER )
#define INLINE __inline
#elif defined(__GNUC__)
#define INLINE __inline__
#else 
#define INLINE
#endif


#define A32D24_1190	       0xf0000000

/*-Macros--------------------------------------------------------*/
/*-command-------------------------------------------------------*/
#define LRS1190_ENABLE(_vmebase){\
				    {\
				       unsigned long _ll, _local;\
				       _ll = _vmebase;\
				       _local = ((_ll) | 0x8002) | A32D24_1190;\
				       *(unsigned short *)_local = 0x1;\
				   }\
				}

/*-command-------------------------------------------------------*/
#define LRS1190_DISABLE(_vmebase){\
				    {\
				       unsigned long _ll, _local;\
				      _ll = _vmebase;\
				       _local = ((_ll) | 0x8002) | A32D24_1190;\
				       *(unsigned short *)_local = 0x0;\
				   }\
				}

/*-command-------------------------------------------------------*/
#define LRS1190_RESET(_vmebase){\
				   {\
				      static unsigned long _ll, _dummy, _local;\
				      _ll = _vmebase;\
				      _local = (_ll) | A32D24_1190;\
				      _dummy = *(unsigned short *) _local;\
				  }\
			       }

/*-command-------------------------------------------------------*/
#define LRS1190_COUNT(_vmebase, _d){\
					{\
					 unsigned long _ll, *_pcount;\
					 _ll = _vmebase;\
					 _pcount = (unsigned long *) (((_ll) | 0x8000) | A32D24_1190);\
					 *((unsigned short *) _d) = *(unsigned short *) _pcount;\
				       }\
				    }


/*-input---------------------------------------------------------*/
#define LRS1190_READI4(_vmebase,_d,_r){\
					{\
					 unsigned long _ll, *_local, __r, _count;\
					 unsigned long *_pcount;\
					 __r = _r;\
					 _ll = _vmebase;\
					 _pcount = (unsigned long *) (((_ll) | 0x8000) | A32D24_1190);\
					 _count = *(unsigned short *) _pcount;\
					 if (_count <= __r)\
					   __r = _count;\
					 _local = (unsigned long *)((_ll)  | A32D24_1190);\
					 while (__r > 0) {\
						*_d++ = *_local++;\
						__r--;}\
					 }\
				      }

/*-input---------------------------------------------------------*/
#define LRS1190_READL2(_vmebase,_d,_r){\
					{\
					 unsigned long _ll, *_pcount, *_local, __r, _count;\
					 __r = _r;\
					 _ll = _vmebase;\
					 _pcount = (unsigned long *) (((_ll) | 0x8000) | A32D24_1190);\
					 _count = *(unsigned short *) _pcount;\
					 if (_count <= __r)\
					   __r = _count;\
					 _local = (unsigned long *)((_ll)  | A32D24_1190);\
					 while (__r > 0) {\
						*(unsigned short *) _d = *(unsigned short *)_local;\
						((unsigned short *)_d)++;\
						(_local)++;\
						__r--;}\
					 }\
				      }

/*-input---------------------------------------------------------*/
#define LRS1190_READH2(_vmebase,_d,_r){\
					{\
					 unsigned long _ll, *_pcount, *_local, _count, __r;\
					 __r = _r;\
					 _ll = _vmebase;\
					 _pcount = (unsigned long *) (((_ll) | 0x8000) | A32D24_1190);\
					 _count = *(unsigned short *) _pcount;\
					 if (_count <= __r)\
					   __r = _count;\
					 _local = (unsigned long *)((_ll)  | A32D24_1190);\
					 while (__r > 0) {\
						*(unsigned short *) _d = *(++((unsigned short *)_local));\
						((unsigned short *)_d)++;\
						((unsigned short *)_local)++;\
						__r--;}\
					 }\
				      }

/*-input---------------------------------------------------------*/
#define LRS_1190_READL2_TONY(_vmebase,_d,_r){\
					{\
					 unsigned long _ll, *_pcount, *_local, __r, _count;\
					 __r = _r;\
					 _ll = _vmebase;\
					 _pcount = (unsigned long *) (((_ll) | 0x8000) | A32D24_1190);\
					 _count = *(unsigned short *) _pcount;\
					 if (_count <= __r)\
					   __r = _count;\
					 _local = (unsigned long *)((_ll)  | A32D24_1190);\
 					 *(unsigned short *) _d = __r;\
					 ((unsigned short *)_d)++;\
					 while (__r > 0) {\
						*(unsigned short *) _d = *(unsigned short *)_local;\
						((unsigned short *)_d)++;\
						(_local)++;\
						__r--;}\
					 }\
				      }
/*---------------------------------------------------------------*/

/*-Functions-----------------------------------------------------*/
INLINE void lrs1190_enable (unsigned long vmeBase)
{
  static unsigned long local;

  local = ((vmeBase) | 0x8002) | A32D24_1190;
  *(unsigned short *)local = 0x1;
}

/*-Functions-----------------------------------------------------*/
INLINE void lrs1190_disable (unsigned long vmeBase)
{
  static unsigned long local;

  local = ((vmeBase) | 0x8002) | A32D24_1190;
  *(unsigned short *)local = 0x0;
}

/*-Functions-----------------------------------------------------*/
INLINE void lrs1190_reset (unsigned long vmeBase)
{
  static unsigned long local, dummy;
  
  local = vmeBase | A32D24_1190;
  dummy = *(unsigned short *)local;
}

/*-Functions-----------------------------------------------------*/
INLINE void lrs1190_count (unsigned long vmeBase, int * count)
{
  static unsigned long local;
  
  local = (vmeBase | 0x8000 | A32D24_1190);
  *count = *(unsigned short *) local;
}

/*-input---------------------------------------------------------*/
INLINE void lrs1190_readi4 (unsigned long vmeBase, unsigned long * d, int r)
{
  static unsigned long local, count;
  
  local = (vmeBase | 0x8000 | A32D24_1190);
  count = *(unsigned short *) local;
  if ((int)count <= r)
    r = count;
  local = vmeBase  | A32D24_1190;
  while (r > 0)
    {
      *d++ = * ((unsigned long *)local++);
      r--;
    }
}

/*-input---------------------------------------------------------*/
INLINE void lrs1190_readl2 (unsigned long vmeBase, unsigned short *d, int r)
{
  static unsigned long local, count;
  
  local = (vmeBase | 0x8000 | A32D24_1190);
  count = *(unsigned short *) local;
  if ((int)count <= r)
    r = count;
  local = vmeBase  | A32D24_1190;
  while (r > 0)
    {
      *d++ = *(unsigned short *)local;
      local++;
      r--;
    }
}

/*-input---------------------------------------------------------*/
INLINE void lrs1190_readh2 (unsigned long vmeBase, unsigned short *d, int r)
{
  static unsigned long local, count;
  
  local = (vmeBase | 0x8000 | A32D24_1190);
  count = *(unsigned short *) local;
  if ((int)count <= r)
    r = count;
  local = vmeBase  | A32D24_1190;
  while (r > 0)
    {
      *d++ = *(++((unsigned short *)local));
      ((unsigned short *)local)++;
      r--;
    }
}

/*-input---------------------------------------------------------*/
INLINE void lrs1190_readl2z (unsigned long vmeBase, unsigned long *d, int r)
{
  static unsigned long local, count;
  
  local = (vmeBase | 0x8000 | A32D24_1190);
  count = *(unsigned short *) local;
  if ((int)count <= r)
    r = count;
  local = vmeBase  | A32D24_1190;
  *(unsigned short *) d = r;
  ((unsigned short *) d)++;
  while (r > 0)
    {
      *(unsigned short *) d = *(unsigned short *)local;
      ((unsigned short *) d)++;
      ((unsigned short *)local)++;
      r--;
    }
}

/*-Tests---------------------------------------------------------*/
void lrs1190(void)
{
  printf("\n---> LRS 1190 Dual port memory buffer (lrs1190.c) <---\n");
  printf("Macro  : LRS1190_ENABLE (unsigned long vmeBase);\n");
  printf("Macro  : LRS1190_DISABLE (unsigned long vmeBase);\n");
  printf("Macro  : LRS1190_CLEAR (unsigned long vmeBase);\n");
  printf("Macro  : LRS1190_COUNT (unsigned long vmeBase, int * count);\n");
  printf("Macro  : LRS1190_READI4 (unsigned long vmeBase, unsigned long *d , int repeat);\n");
  printf("Macro  : LRS1190_READL2 (unsigned long vmeBase, unsigned short *d , int repeat);\n");
  printf("Macro  : LRS1190_READH2 (unsigned long vmeBase, unsigned short *d , int repeat);\n");
  printf("Macro  : LRS1190_READL2_TONY (unsigned long vmeBase, unsigned long *d , int repeat);\n");
  printf("Inline : lrs1190_enable (unsigned long vmeBase);\n");
  printf("Inline : lrs1190_disable (unsigned long vmeBase);\n");
  printf("Inline : lrs1190_clear (unsigned long vmeBase);\n");
  printf("Inline : lrs1190_count (unsigned long vmeBase, int * count);\n");
  printf("Inline : lrs1190_readi4 (unsigned long vmeBase, unsigned long *d , int repeat);\n");
  printf("Inline : lrs1190_readl2 (unsigned long vmeBase, unsigned long *d , int repeat);\n");
  printf("Inline : lrs1190_readh2 (unsigned long vmeBase, unsigned long *d , int repeat);\n");
  printf("Inline : lrs1190_readl2_tony (unsigned long vmeBase, unsigned long *d , int repeat);\n");
  printf("Test : ena1190  (0x7f0000)         <--- enable  VME buffer\n");
  printf("Test : dis1190  (0x7f0000)         <--- disable VME buffer\n");
  printf("Test : res1190  (0x7f0000)         <--- reset   VME buffer\n");
  printf("Test : cn1190   (0x7f0000)         <--- read    unsigned short count register\n");
  printf("Test : rd1190   (0x7f0000)         <--- read             VME buffer\n");
  printf("Test : rd1190l  (0x7f0000)         <--- read low  unsigned short   VME buffer\n");
  printf("Test : rd1190h  (0x7f0000)         <--- read high unsigned short   VME buffer\n");
}

void ena1190 (unsigned long vmeBase)
{
  LRS1190_ENABLE (vmeBase);
}

void dis1190 (unsigned long vmeBase)
{
  LRS1190_DISABLE (vmeBase);
}

void res1190 (unsigned long vmeBase)
{
  LRS1190_RESET (vmeBase);
}

void cn1190 (unsigned long vmeBase)
{
  int count;
  lrs1190_count(vmeBase, &count);
  printf("Inline counter %i\n",count);
  LRS1190_COUNT(vmeBase, &count);
  printf("Macro counter %i\n",count);
}

void rd1190l ( unsigned long vmeBase , unsigned long r)
{
  unsigned long i, dd[1024], *pdd;
  unsigned short buf_counter;
  for (i=0;i<1024;i++)
    dd[i] = 0;
  pdd = &dd[0];
  LRS1190_DISABLE (vmeBase);
  LRS1190_COUNT (vmeBase, &buf_counter);
  printf("counter %i\n",buf_counter);
  LRS1190_READL2 (vmeBase, pdd , r);
  LRS1190_ENABLE (vmeBase);
  for (i=0; i<buf_counter ;i++)
    printf("%i-> %8.8x\n",i,dd[i]);
}

void rd1190h ( unsigned long vmeBase , unsigned long r)
{
  unsigned long i, dd[1024], *pdd;
  unsigned short buf_counter;
  for (i=0;i<1024;i++)
    dd[i] = 0;
  pdd = &dd[0];
  LRS1190_DISABLE (vmeBase);
  LRS1190_COUNT (vmeBase, &buf_counter);
  LRS1190_READH2 (vmeBase, pdd , r);
  LRS1190_ENABLE (vmeBase);
  for (i=0; i<buf_counter ;i++)
    printf("%i-> %8.8x\n",i,dd[i]);
}

void rd1190 ( unsigned long vmeBase , unsigned long r)
{
  unsigned long i, dd[1024], *pdd;
  unsigned short buf_counter;
  for (i=0;i<1024;i++)
    dd[i] = 0;
  pdd = &dd[0];
  LRS1190_DISABLE (vmeBase);
  LRS1190_COUNT (vmeBase, &buf_counter);
  LRS1190_READI4 (vmeBase, pdd, r);
  LRS1190_ENABLE (vmeBase);
  for (i=0; i<buf_counter ;i++)
    printf("%i-> %8.8x\n",i,dd[i]);
}
