/*-----------------------------------------------------------------------------
 *  Copyright (c) 1998      TRIUMF Data Acquistion Group
 *  Please leave this header in any reproduction of that distribution
 * 
 *  TRIUMF Data Acquisition Group, 4004 Wesbrook Mall, Vancouver, B.C. V6T 2A3
 *  Email: online@triumf.ca           Tel: (604) 222-1047  Fax: (604) 222-1074
 *         amaudruz@triumf.ca
 * -----------------------------------------------------------------------------
 *  
 *  Description	: CAMAC interface for CES8210 CAMAC controller using
 *		  Midas Camac Standard calls. includes the interrupt handling.
 *		  No reference to Midas.
 *                interrupt level 2 INT2 at vector  85  <---- not implemented
 *                interrupt level 4 INT4 at vector 170  <---- current setting
 *
 *  Requires 	: vxWorks lib calls for the interrupts
 *
 *  Application : VME + VxWorks
 *
 *  Author:  Pierre-Andre Amaudruz Data Acquisition Group
 * 
 *  Revision 1.0  1998        Pierre	 Initial revision
 *
 *---------------------------------------------------------------------------*/

#ifndef INLINE
#if defined( _MSC_VER )
#define INLINE __inline
#elif defined(__GNUC__)
#define INLINE __inline__
#else 
#define INLINE
#endif
#endif

INLINE void cam16i(const int c, const int n, const int a, const int f, unsigned short *d);
INLINE void cam24i(const int c, const int n, const int a, const int f, unsigned long *d);
INLINE void cam16i_q(const int c, const int n, const int a, const int f, unsigned short *d, int *x, int *q);
INLINE void cam24i_q(const int c, const int n, const int a, const int f, unsigned long *d, int *x, int *q);
INLINE void cam16i_r(const int c, const int n, const int a, const int f, unsigned short **d, const int r);
INLINE void cam24i_r(const int c, const int n, const int a, const int f, unsigned long **d, const int r);
INLINE void cam16i_rq(const int c, const int n, const int a, const int f, unsigned short **d, const int r);
INLINE void cam24i_rq(const int c, const int n, const int a, const int f, unsigned long **d, const int r);
INLINE void cam16i_sa(const int c, const int n, const int a, const int f, unsigned short **d, const int r);
INLINE void cam24i_sa(const int c, const int n, const int a, const int f, unsigned long **d, const int r);
INLINE void cam16i_sn(const int c, const int n, const int a, const int f, unsigned short **d, const int r);
INLINE void cam24i_sn(const int c, const int n, const int a, const int f, unsigned long **d, const int r);
INLINE void cami(const int c, const int n, const int a, const int f, unsigned short *d);
INLINE void cam16o(const int c, const int n, const int a, const int f, unsigned short d);
INLINE void cam24o(const int c, const int n, const int a, const int f, unsigned long d);
INLINE void cam16o_q(const int c, const int n, const int a, const int f, unsigned short d, int *x, int *q);
INLINE void cam24o_q(const int c, const int n, const int a, const int f, unsigned long d, int *x, int *q);
INLINE void cam16o_r(const int c, const int n, const int a, const int f, unsigned short *d, const int r);
INLINE void cam24o_r(const int c, const int n, const int a, const int f, unsigned long *d, const int r);
INLINE void camo(const int c, const int n, const int a, const int f, unsigned short d);
INLINE int camc_chk(const int c);
INLINE void camc(const int c, const int n, const int a, const int f);
INLINE void camc_q(const int c, const int n, const int a, const int f, int *q);
INLINE void camc_sa(const int c, const int n, const int a, const int f, const int r);
INLINE void camc_sn(const int c, const int n, const int a, const int f, const int r);
INLINE int cam_init(void);
INLINE void cam_exit(void);
INLINE void cam_inhibit_set(const int c);
INLINE void cam_inhibit_clear(const int c);
INLINE void cam_crate_clear(const int c);
INLINE void cam_crate_zinit(const int c);
INLINE void cam_lam_enable(const int c, const int n);
INLINE void cam_lam_disable(const int c, const int n);
INLINE void cam_lam_read(const int c, int *lam);
INLINE void cam_lam_clear(const int c, const int n);
INLINE void came_cn(int *ext, const int b, const int c, const int n, const int a);
INLINE void came_ext(const int ext, int *b, int *c, int *n, int *a);
INLINE void cam16ei(const int ext, const int f, unsigned short *d);
INLINE void cam24ei(const int ext, const int f, unsigned long *d);
INLINE void cam16ei_q(const int ext, const int f, unsigned short *d, int *x, int *q);
INLINE void cam24ei_q(const int ext, const int f, unsigned long *d, int *x, int *q);
INLINE void cam16ei_r(const int ext, const int f, unsigned short **d, const int r);
INLINE void cam24ei_r(const int ext, const int f, unsigned long **d, const int r);
INLINE void cam16ei_rq(const int ext, const int f, unsigned short **d, const int r);
INLINE void cam24ei_rq(const int ext, const int f, unsigned long **d, const int r);
INLINE void cam16ei_saq(const int ext, const int f, unsigned short **d, const int r);
INLINE void cam24ei_saq(const int ext, const int f, unsigned long **d, const int r);
INLINE void cam16ei_snq(const int ext, const int f, unsigned short **d, const int r);
INLINE void cam24ei_snq(const int ext, const int f, unsigned long **d, const int r);
INLINE void cam16eo(const int ext, const int f, unsigned short d);
INLINE void cam24eo(const int ext, const int f, unsigned long d);
INLINE void cam16eo_q(const int ext, const int f, unsigned short d, int *x, int *q);
INLINE void cam24eo_q(const int ext, const int f, unsigned long d, int *x, int *q);
INLINE void camec(const int ext, const int f);
INLINE void camec_q(const int ext, const int f, int *x, int *q);
INLINE void camec_sa(const int ext, const int f, const int r);
INLINE void camec_sn(const int ext, const int f, const int r);
INLINE void cam_interrupt_enable(void);
INLINE void cam_interrupt_disable(void);
INLINE void cam_interrupt_attach(void (*isr)());
INLINE void cam_interrupt_detach(void);
